import requests
from deso.utils import submitTransaction
import requests
from deso.Sign import Sign_Transaction


class Derived:
    # takes two optional Argument; publicKey and nodeURL. By default NodeURL is https://node.deso.org/api/v0/"
    def __init__(self, nodeURL="https://node.deso.org/api/v0/",  minFee=1000):
        self.NODE_URL = nodeURL
        self.MIN_FEE = minFee

    def authorizeDerivedKey(self, publicKey, derivedPublicKey, derivedSeedHex, expirationBlock, accessSignature, transactionSpendingLimitHex, derivedKeySignature=True, isAuth=True):
        try:
            error = None
            payload = {
                "OwnerPublicKeyBase58Check": publicKey,
                "DerivedPublicKeyBase58Check": derivedPublicKey,
                "ExpirationBlock": expirationBlock,
                "AccessSignature": accessSignature,
                "DeleteKey": not isAuth,
                "DerivedKeySignature": derivedKeySignature,
                "transactionSpendingLimitHex": transactionSpendingLimitHex,
                "MinFeeRateNanosPerKB": self.MIN_FEE,

            }
            endpoint = self.NODE_URL + "authorize-derived-key"
            response = requests.post(endpoint, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            try:
                signedTransactionHex = Sign_Transaction(
                    derivedSeedHex, transactionHex
                )  # txn signature
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure public key, derived key and derivedSeedHex are correct.\nAlso, Make sure the derived Key hasAUTHORIZE_DERIVED_KEY permission as well"}
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL)
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])
    


from urllib import response
import requests


class Posts:
    def __init__(self,  nodeURL="https://node.deso.org/api/v0/"):
        self.NODE_URL = nodeURL

    def getPostsStateless(self, readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop", addGlobalFeedBool=False, fetchSubcommnets=False, getPostsByDESO=False, getPostsForFollowFeed=False, getPostsForGlobalWhitelist=True, mediaRequired=False, numToFetch=50, orderBy="", postContent="", postHashHex="", postsByDESOMinutesLookBack=0, customPayload={}):
        endpointURL = self.NODE_URL + "get-posts-stateless"
        payload = {"PostHashHex": postHashHex,
                   "ReaderPublicKeyBase58Check": readerPublicKey,
                   "OrderBy": orderBy,
                   "StartTstampSecs": None,
                   "PostContent": postContent,
                   "NumToFetch": numToFetch,
                   "FetchSubcomments": fetchSubcommnets,
                   "GetPostsForFollowFeed": getPostsForFollowFeed,
                   "GetPostsForGlobalWhitelist": getPostsForGlobalWhitelist,
                   "GetPostsByDESO": getPostsByDESO,
                   "MediaRequired": mediaRequired,
                   "PostsByDESOMinutesLookback": postsByDESOMinutesLookBack,
                   "AddGlobalFeedBool": addGlobalFeedBool}
        response = requests.post(endpointURL, json=payload)
        return response

    def getSinglePost(self, postHashHex, readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop", fetchParents=False, commnetLimit=10, commentOffset=0, addGlobalFeedBool=False, loadAuthorThread=True, ThreadLeafLimit=1, ThreadLevelLimit=2):
        endpointURL = self.NODE_URL + "get-single-post"
        payload = {"PostHashHex": postHashHex,
                   "ReaderPublicKeyBase58Check": readerPublicKey,
                   "FetchParents": fetchParents,
                   "CommentOffset": commentOffset,
                   "CommentLimit": commnetLimit,
                   "AddGlobalFeedBool": addGlobalFeedBool,
                   "ThreadLevelLimit": ThreadLevelLimit,
                   "ThreadLeafLimit": ThreadLeafLimit,
                   "LoadAuthorThread": loadAuthorThread}
        response = requests.post(endpointURL, json=payload)
        return response

    def getPostsForPublicKey(self, username="", publicKey="", mediaRequired=False, numToFetch=10, lastPostHashHex="", readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop"):
        endpoint = self.NODE_URL + "get-posts-for-public-key"
        payload = {"PublicKeyBase58Check": publicKey,
                   "Username": username,
                   "ReaderPublicKeyBase58Check": readerPublicKey,
                   "LastPostHashHex": lastPostHashHex,
                   "NumToFetch": numToFetch,
                   "MediaRequired": mediaRequired}
        response = requests.post(endpoint, json=payload)
        return response

    def getDiamondsForPost(self, postHashHex, limit=25, offset=0, readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop"):
        endpointURL = self.NODE_URL + "get-diamonds-for-post"
        payload = {"PostHashHex": postHashHex,
                   "Offset": offset,
                   "Limit": limit,
                   "ReaderPublicKeyBase58Check": readerPublicKey}
        response = requests.post(endpointURL, json=payload)
        return response

    def getLikesForPost(self, postHashHex, limit=50, offset=0, readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop"):
        endpointURL = self.NODE_URL + "get-likes-for-post"
        payload = {"PostHashHex": postHashHex,
                   "Offset": offset,
                   "Limit": limit,
                   "ReaderPublicKeyBase58Check": readerPublicKey}
        response = requests.post(endpointURL, json=payload)
        return response

    def getQuoteRepostsForPost(self, postHashHex, limit=50, offset=0, readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop"):
        endpointURL = self.NODE_URL + "get-quote-reposts-for-post"
        payload = {"PostHashHex": postHashHex,
                   "Offset": offset,
                   "Limit": limit,
                   "ReaderPublicKeyBase58Check": readerPublicKey}
        response = requests.post(endpointURL, json=payload)
        return response

    def getNFTEntriesForNFTPost(self, postHashHex, readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop"):
        endpointURL = self.NODE_URL + "get-nft-entries-for-nft-post"
        payload = {"ReaderPublicKeyBase58Check": readerPublicKey,
                   "PostHashHex": postHashHex}
        response = requests.post(endpointURL, json=payload)
        return response

    def getHotFeed(self, taggedUsername, responeLimit=10, sortByNew=True, seenPosts=[], readerPublicKey="BC1YLgk64us61PUyJ7iTEkV4y2GqpHSi8ejWJRnZwsX6XRTZSfUKsop"):
        '''Returns posts that has mentioned in username'''
        endpointURL = self.NODE_URL + "get-hot-feed"
        payload = {"ReaderPublicKeyBase58Check":
                   readerPublicKey,
                   "SeenPosts": seenPosts,
                   "Tag": f"@{taggedUsername}",
                   "SortByNew": sortByNew,
                   "ResponseLimit": responeLimit}

        response = requests.post(endpointURL, json=payload)
        return response



from deso.utils import getUserJWT
import requests


class Metadata:
    def __init__(self, publicKey="BC1YLhBLE1834FBJbQ9JU23JbPanNYMkUsdpJZrFVqNGsCe7YadYiUg", nodeURL="https://node.deso.org/api/v0/"):
        self.PUBLIC_KEY = publicKey
        self.NODE_URL = nodeURL

    def getNodeHealth(self):
        endpointURL = self.NODE_URL + "health-check"
        response = requests.get(endpointURL)
        return response

    def getExchangeRate(self):
        endpointURL = self.NODE_URL + "get-exchange-rate"
        response = requests.get(endpointURL)
        return response

    def getAppState(self):
        payload = {"PublicKeyBase58Check": self.PUBLIC_KEY}

        endpointURL = self.NODE_URL + "get-app-state"
        response = requests.post(endpointURL, json=payload)
        return response

    def getDiamondLevelMap(self, inDesoNanos = True):
        if inDesoNanos:
            return {
                "1": 50000, "2": 500000, "3": 5000000, "4": 50000000, "5": 500000000, "6": 5000000000, "7": 50000000000, "8": 500000000000
            }
        
        else:
            return{
             "1": 0.00005, "2": 0.0005, "3": 0.005, "4": 0.05, "5": 0.5, "6": 5, "7": 50, "8": 500
        }



from deso.utils import submitTransaction, appendExtraData
import requests
from deso.Sign import Sign_Transaction


class Trade:
    def __init__(self, publicKey, seedHex=None,  nodeURL="https://node.deso.org/api/v0/", derivedPublicKey=None, derivedSeedHex=None, minFee=1000, derivedKeyFee=1700):
        self.SEED_HEX = seedHex
        self.PUBLIC_KEY = publicKey
        self.NODE_URL = nodeURL
        self.DERIVED_PUBLIC_KEY = derivedPublicKey
        self.DERIVED_SEED_HEX = derivedSeedHex
        self.MIN_FEE = minFee if seedHex else derivedKeyFee

    def sendDeso(self, recieverPublicKeyOrUsername, desoToSend):
        try:
            error = None
            endpointURL = self.NODE_URL + "send-deso"
            payload = {"SenderPublicKeyBase58Check": self.PUBLIC_KEY,
                       "RecipientPublicKeyOrUsername": recieverPublicKeyOrUsername,
                       "AmountNanos": int(round(desoToSend*1e9)),
                       "MinFeeRateNanosPerKB": self.MIN_FEE}
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if self.DERIVED_PUBLIC_KEY is not None and self.DERIVED_SEED_HEX is not None and self.SEED_HEX is None:
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL)
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex)
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."}
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL)
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def buyCreatorCoin(self, creatorPublicKey, desoAmountToBuy):
        try:
            error = None
            endpointURL = self.NODE_URL + "buy-or-sell-creator-coin"
            payload = {"UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                       "CreatorPublicKeyBase58Check": creatorPublicKey,
                       "OperationType": "buy",
                       "DeSoToSellNanos": int(desoAmountToBuy * 1e9),
                       "CreatorCoinToSellNanos": 0,
                       "DeSoToAddNanos": 0,
                       "MinDeSoExpectedNanos": 0,
                       "MinCreatorCoinExpectedNanos": 0,
                       "MinFeeRateNanosPerKB": self.MIN_FEE,
                       "InTutorial": False}
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if self.DERIVED_PUBLIC_KEY is not None and self.DERIVED_SEED_HEX is not None and self.SEED_HEX is None:
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL)
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex)
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."}
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL)
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def getHeldCoinsOfCreator(self, publicKeyOfCoin):
        endpoint = self.NODE_URL + "get-users-stateless"
        payload = {"PublicKeysBase58Check": [self.PUBLIC_KEY]}
        response = requests.post(endpoint, json=payload)
        hodlings = response.json()["UserList"][0]["UsersYouHODL"]
        for hodling in hodlings:
            if hodling["CreatorPublicKeyBase58Check"] == publicKeyOfCoin:
                coinsHeld = hodling["BalanceNanos"]
                if coinsHeld != 0:
                    return hodling["BalanceNanos"]
                else:
                    return -1
        return -1

    def amountOnSell(desoLockedNanos, coinsInCirculation, balanceNanos):
        beforeFees = desoLockedNanos * \
            (1 - pow((1-balanceNanos/coinsInCirculation), (1 / 0.3333333)))
        return ((beforeFees * (100*100 - 1)) / (100*100))

    def sellCreatorCoin(self, creatorPublicKey, coinsToSellNanos):
        try:
            endpointURL = self.NODE_URL + "buy-or-sell-creator-coin"
            payload = {
                "UpdaterPublicKeyBase58Check": self.PUBLIC_KEY,
                "CreatorPublicKeyBase58Check": creatorPublicKey,
                "OperationType": "sell",
                "DeSoToSellNanos": 0,
                "CreatorCoinToSellNanos": coinsToSellNanos,
                "DeSoToAddNanos": 0,
                "MinDeSoExpectedNanos": 0,
                "MinCreatorCoinExpectedNanos": 0,
                "MinFeeRateNanosPerKB": 1000,
            }
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if self.DERIVED_PUBLIC_KEY is not None and self.DERIVED_SEED_HEX is not None and self.SEED_HEX is None:
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL)
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex)
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."}
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL)
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def sendCreatorCoins(self, creatorPublicKey, receiverUsernameOrPublicKey, creatorCoinNanosToSend):
        try:
            error = None
            endpointURL = self.NODE_URL + "transfer-creator-coin"
            payload = {"SenderPublicKeyBase58Check": self.PUBLIC_KEY,
                       "CreatorPublicKeyBase58Check": creatorPublicKey,
                       "ReceiverUsernameOrPublicKeyBase58Check": receiverUsernameOrPublicKey,
                       "CreatorCoinToTransferNanos": creatorCoinNanosToSend,
                       "MinFeeRateNanosPerKB": self.MIN_FEE}
            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if self.DERIVED_PUBLIC_KEY is not None and self.DERIVED_SEED_HEX is not None and self.SEED_HEX is None:
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL)
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex)
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."}
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL)
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])

    def sendDAOCoins(self,  coinsToTransfer, daoPublicKeyOrName, receiverPublicKeyOrUsername):
        '''Sends DAO coin to publicKey or username. Use the hex() function to convert a number to hexadecimal
        for Example, if you want to send 15 DAO coin, set coinsToTransferNanosInHex to hex(int(15*1e18))'''
        try:
            error = None
            endpointURL = self.NODE_URL + "transfer-dao-coin"
            payload = {"SenderPublicKeyBase58Check": self.PUBLIC_KEY,
                       "ProfilePublicKeyBase58CheckOrUsername": daoPublicKeyOrName,
                       "ReceiverPublicKeyBase58CheckOrUsername": receiverPublicKeyOrUsername,
                       "DAOCoinToTransferNanos": str(coinsToTransfer),
                       "MinFeeRateNanosPerKB": self.MIN_FEE}

            response = requests.post(endpointURL, json=payload)
            error = response.json()
            transactionHex = response.json()["TransactionHex"]
            if self.DERIVED_PUBLIC_KEY is not None and self.DERIVED_SEED_HEX is not None and self.SEED_HEX is None:
                extraDataResponse = appendExtraData(
                    transactionHex, self.DERIVED_PUBLIC_KEY, self.NODE_URL)
                error = extraDataResponse.json()
                transactionHex = extraDataResponse.json()["TransactionHex"]
            seedHexToSignWith = self.SEED_HEX if self.SEED_HEX else self.DERIVED_SEED_HEX
            try:
                signedTransactionHex = Sign_Transaction(
                    seedHexToSignWith, transactionHex)
            except Exception as e:
                error = {
                    "error": "Something went wrong while signing the transactions. Make sure publicKey and seedHex are correct."}
            submitTransactionResponse = submitTransaction(
                signedTransactionHex, self.NODE_URL)
            return submitTransactionResponse
        except Exception as e:
            raise Exception(error["error"])



from deso.Identity import Identity
from deso.Media import Media
from deso.Metadata import Metadata
from deso.Social import Social
from deso.Derived import Derived
from deso.Posts import Posts
from deso.Trade import Trade
from deso.User import User


# This file has code for signing a transaction --> Credit: MiniGlome
import hashlib
import hmac

def inverse_mod(k, p):
    """Returns the inverse of k modulo p.
    This function returns the only integer x such that (x * k) % p == 1.
    k must be non-zero and p must be a prime.
    """
    if k == 0:
        raise ZeroDivisionError('division by zero')

    if k < 0:
        # k ** -1 = p - (-k) ** -1  (mod p)
        return p - inverse_mod(-k, p)

    # Extended Euclidean algorithm.
    s, old_s = 0, 1
    t, old_t = 1, 0
    r, old_r = p, k

    while r != 0:
        quotient = old_r // r
        old_r, r = r, old_r - quotient * r
        old_s, s = s, old_s - quotient * s
        old_t, t = t, old_t - quotient * t

    gcd, x, y = old_r, old_s, old_t

    return x % p

def get_hmac(key, data):
    return hmac.new(key, data, hashlib.sha256).digest()


def hmac_drbg(entropy, string):
    material = entropy + string
    K = b"\x00" * 32
    V = b"\x01" * 32

    K = get_hmac(K, V + b"\x00" + material)
    V = get_hmac(K, V)
    K = get_hmac(K, V + b"\x01" + material)
    V = get_hmac(K, V)

    temp = b""
    while len(temp) < 32:
        V = get_hmac(K, V)
        temp += V

    return temp[:32]

#######


g = (0x79be667ef9dcbbac55a06295ce870b07029bfcdb2dce28d959f2815b16f81798,
     0x483ada7726a3c4655da4fbfc0e1108a8fd17b448a68554199c47d08ffb10d4b8)
p = 0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffefffffc2f
n = 0xfffffffffffffffffffffffffffffffebaaedce6af48a03bbfd25e8cd0364141


def point_add(point1, point2):
    # Returns the result of point1 + point2 according to the group law.
    if point1 is None:
        return point2
    if point2 is None:
        return point1

    x1, y1 = point1
    x2, y2 = point2

    if x1 == x2 and y1 != y2:
        return None

    if x1 == x2:
        m = (3 * x1 * x1) * inverse_mod(2 * y1, p)
    else:
        m = (y1 - y2) * inverse_mod(x1 - x2, p)

    x3 = m * m - x1 - x2
    y3 = y1 + m * (x3 - x1)
    result = (x3 % p, -y3 % p)

    return result


def scalar_mult(k, point):
    # Returns k * point computed using the double and point_add algorithm.
    result = None
    addend = point

    while k:
        if k & 1:
            # Add.
            result = point_add(result, addend)
        # Double.
        addend = point_add(addend, addend)
        k >>= 1

    return result

#######


def to_DER(r, s):  # Signature to DER format
    r = bytes.fromhex(r)
    s = bytes.fromhex(s)
    if r[0] >= 0x80:
        r = bytes.fromhex("00")+r
    if s[0] >= 0x80:
        s = bytes.fromhex("00")+s
    res = bytes.fromhex("02"+hex(len(r))[2:]) + \
        r + bytes.fromhex("02"+hex(len(s))[2:]) + s
    res = bytes.fromhex("30"+hex(len(res))[2:]) + res

    return res.hex()


def hexify(n):
    n = hex(n)[2:]
    if len(n) % 2 != 0:
        n = "0"+n
    return n


def Sign_Transaction(seedHex, TransactionHex):
    s256 = hashlib.sha256(hashlib.sha256(
        bytes.fromhex(TransactionHex)).digest()).digest()
    drbg = hmac_drbg(entropy=bytes.fromhex(seedHex), string=s256)
    k = int.from_bytes(drbg, 'big')
    kp = scalar_mult(k, g)
    kpX = kp[0]
    r = kpX % n
    s = inverse_mod(k, n) * (r * int(seedHex, 16)+int(s256.hex(), 16))
    s = s % n
    signature = to_DER(hexify(r), hexify(s))
    signed_transaction = TransactionHex[:-2] + \
        hex(len(bytearray.fromhex(signature)))[2:] + signature

    return signed_transaction



from urllib import response
import requests


class User:
    def __init__(self,  nodeURL="https://node.deso.org/api/v0/"):
        self.NODE_URL = nodeURL

    def getSingleProfile(self, username="", publicKey="", NoErrorOnMissing=False):
        endpointURL = self.NODE_URL + "get-single-profile"
        payload = {"PublicKeyBase58Check": publicKey,
                   "Username": username, "NoErrorOnMissing": NoErrorOnMissing}
        response = requests.post(endpointURL, json=payload)
        return response

    def getUsersStateless(self, listOfPublicKeys, skipForLeaderboard=True):
        endpointURL = self.NODE_URL + "get-users-stateless"
        payload = {"PublicKeysBase58Check": listOfPublicKeys,
                   "SkipForLeaderboard": skipForLeaderboard}
        response = requests.post(endpointURL, json=payload)
        return response

    def getProfilePicURL(self, publicKey):
        profilePicURL = f'{self.NODE_URL}get-single-profile-picture/{publicKey}?fallback=https://node.deso.org/assets/img/default_profile_pic.png'
        return profilePicURL

    def getMessagesStateless(self, publicKey, numToFetch=25, sortAlgorithm="time", followersOnly=False, followingOnly=False, holdersOnly=False, holdingsOnly=False, fetchAfterPublicKey=""):
        endpointURL = self.NODE_URL + "get-messages-stateless"
        payload = {"PublicKeyBase58Check": publicKey,
                   "FetchAfterPublicKeyBase58Check": fetchAfterPublicKey,
                   "NumToFetch": numToFetch,
                   "HoldersOnly": holdersOnly,
                   "HoldingsOnly": holdingsOnly,
                   "FollowersOnly": followersOnly,
                   "FollowingOnly": followingOnly,
                   "SortAlgorithm": sortAlgorithm}
        response = requests.post(endpointURL, json=payload)
        return response

    def getNotifications(self, publicKey, startIndex=-1, numToFetch=50, filterOutNotificationCategories={}):
        # filterOutNotification is a map that looks like {"diamond": True, "like": True,  "transfer": True, "follow": True, "nft": True, "post": True}
        # ever True means that the specific notification category will be filtered out
        payload = {"PublicKeyBase58Check": publicKey,
                   "FetchStartIndex": startIndex,
                   "NumToFetch": numToFetch,
                   "FilteredOutNotificationCategories": filterOutNotificationCategories}
        endpointURL = "https://diamondapp.com/api/v0/get-notifications"
        response = requests.post(endpointURL, json=payload)
        return response

    def getNFTs(self, userPublicKey, readerPublicKey="", isForSale=False):
        '''Gets the NFTs associated with a user,
            setting isForSale = True returns only the NFTs that are for sale.'''
        payload = {"UserPublicKeyBase58Check": userPublicKey,
                   "ReaderPublicKeyBase58Check": readerPublicKey,
                   "IsForSale": isForSale}

        endpointURL = self.NODE_URL + "get-nfts-for-user"
        response = requests.post(endpointURL, json=payload)
        return response

    def getDerivedKeys(self, publicKey):
        payload = {"PublicKeyBase58Check": publicKey}
        endpointURL = self.NODE_URL + "get-user-derived-keys"
        response = requests.post(endpointURL, json=payload)
        return response

    def getTransactionInfo(self, publicKey, limit=200, lastTransactionIDBase58Check="", lastPublicKeyTransactionIndex=-1):
        payload = {"PublicKeyBase58Check": publicKey,
                   "LastTransactionIDBase58Check": lastTransactionIDBase58Check,
                   "LastPublicKeyTransactionIndex": lastPublicKeyTransactionIndex, "Limit": limit}
        endpointURL = self.NODE_URL + "transaction-info"
        response = requests.post(endpointURL, json=payload)
        return response

    def getHoldersForPublicKey(self, publicKey="", username="", fetchAll=False, numToFetch=100, fetchHodlings=False, isDAOCOIN=False, lastPublicKey=""):
        payload = {"PublicKeyBase58Check": publicKey,
                   "Username": username,
                   "LastPublicKeyBase58Check": lastPublicKey,
                   "NumToFetch": numToFetch,
                   "FetchHodlings": fetchHodlings,
                   "FetchAll": fetchAll,
                   "IsDAOCoin": isDAOCOIN}

        endpointURL = self.NODE_URL + "get-hodlers-for-public-key"
        response = requests.post(endpointURL, json=payload)
        return response

    def getFollowsStateless(self, username="", publicKey="", getFollowing=True, numToFetch=50, lastPublicKey=""):
        '''Returns list of user followings and followers'''
        payload = {"Username": username,
                   "PublicKeyBase58Check": publicKey,
                   "GetEntriesFollowingUsername": not getFollowing,
                   "LastPublicKeyBase58Check": lastPublicKey,
                   "NumToFetch": numToFetch}

        endpointURL = self.NODE_URL + "get-follows-stateless"
        response = requests.post(endpointURL, json=payload)
        return response

    def getDaoCoinLimitOrders(self, daoCoinPublicKey):
        payload = {"DAOCoin1CreatorPublicKeyBase58Check": daoCoinPublicKey,
                   "DAOCoin2CreatorPublicKeyBase58Check": "DESO"}
        endpointURL = self.NODE_URL + "get-dao-coin-limit-orders"
        response = requests.post(endpointURL, json=payload)
        return response
